<?php
namespace Teknisa\MAN\Utilities;

use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Row;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DataSource\FilterCriteria;
class Util extends \Zeedhi\Framework\Controller\Simple {

	protected $instanceManager;
    protected $entityManager;
    protected $salesMessage;
    
	public function __construct(\Zeedhi\Framework\DependencyInjection\InstanceManager $instanceManager, SalesMessage $salesMessage, \Doctrine\ORM\EntityManager $entityManager) {
		$this->instanceManager = $instanceManager;
		$this->salesMessage  = $salesMessage;
        $this->entityManager = $entityManager;
	}	
	
    protected static function getSalt() {
        return str_pad("zeedhi", 32, "\0");
    }

    public static function moneyToDouble($value) {
		if (!!$value) {
			$value = preg_replace("/[^0-9,]/", "", $value);
			$value = str_replace ("," , "." , $value);			
		}
		else {
			$value = 0;
		}
		
		return floatval($value);
	}

    /**
     * Prepares an SQL statement.
     *
     * @param string $query The Query.
     * @param string $arrName The variable to be replaced.
     * @param array  $arrValue The array of things to be placed on string.
     * @return string The prepared Query.
     *
     */	
	public function putArrayOnQuery($query, $arrName, $arrValue) {
		for ($i=0; $i < sizeof($arrValue); $i++) {
            if(key($arrValue) === 0) {
                $arrValue[$i] = "'" . $arrValue[$i];
            }
            if(end($arrValue) == $arrValue[$i]) {
                $arrValue[$i] = $arrValue[$i] . "'";
            }
			$arrValue[$i] = $arrValue[$i];
		}
		$query = preg_replace('/'.$arrName.'/', implode(", ", $arrValue), $query, 1);
		return $query;
	}
	
	public function putArrayOnQueryPipe($query, $arrName, $arrValue) {
        for ($i=0; $i < sizeof($arrValue); $i++) {
            if(key($arrValue) === 0) {
                $arrValue[$i] = "'|" . $arrValue[$i];
            }
            if(end($arrValue) == $arrValue[$i]) {
                $arrValue[$i] = $arrValue[$i] . "|'";
            }
            $arrValue[$i] = $arrValue[$i];
        }
        $query = preg_replace('/'.$arrName.'/', implode("|", $arrValue), $query, 4);
        return $query;
    }

	public function testConnection(Request\Filter $request, Response $response) {
        try {
			$response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('UtilitiesTestConnectionRepo', array(array('nothing' => 'nothing'))));
        } catch (\Exception $e) {            
            $response->addMessage(new Message($e->getMessage()));
        }
    }

    public function cleanSpecialChar($string) {
       $string = str_replace('-','',$string); 
       return preg_replace('/[^A-Za-z0-9\-]/', '', $string); // Removes special chars.
    }

    public function checkVersion($frontVersion) {
        $backVersion = $this->instanceManager->getParameter('backVersion');

        if ($backVersion === $frontVersion) {
        	$versionOk = true;	
        } else {
        	$versionOk = false;	
        } 

        return array(
            'versionOk' => $versionOk,
            'frontVersion' => $frontVersion,
            'backVersion' => $backVersion
        );
    }    

    public function compararMenorData($dataAtual, $data) {
        $ano1 = substr($dataAtual, 4,8);
        $mes1 = substr($dataAtual, 2,2);
        $dia1 = substr($dataAtual, 0,2);

        $ano2 = substr($data, 4,8);
        $mes2 = substr($data, 2,2);
        $dia2 = substr($data, 0,2);

        if($ano2 < $ano1) {
            return false;
        }else if($ano2 == $ano1 && $mes2 < $mes1) {
            return false;
        }else if($ano2 == $ano1 && $mes2 == $mes1 && $dia2 < $dia1) {
            return false;
        }else{
            return true;
        }
    }

    public function replaceParams($params) {
        for($i = 0; $i < count($params); ++$i) {            
           if($params[$i]["value"] == '=') {
             $params[$i]["value"] = '';
           }
        }
        return $params;
    }

    public static function encrypt($text, $salt = 'setforms') {        
        return md5('odhen'.$text.'7833');
    }

    public static function oldEncrypt($text) {
        return trim(
            base64_encode(
                mcrypt_encrypt(
                    MCRYPT_RIJNDAEL_256,
                    self::getSalt(),
                    $text,
                    MCRYPT_MODE_ECB,
                    mcrypt_create_iv(mcrypt_get_iv_size(MCRYPT_RIJNDAEL_256, MCRYPT_MODE_ECB), MCRYPT_RAND)
                )
            )
        );
    }

    /**
     * Decrypt.
     *
     * @param string $text Text.
     *
     * @return string
     */
    public static function decrypt($text) {
        return trim(
            mcrypt_decrypt(
                MCRYPT_RIJNDAEL_256,
                self::getSalt(),
                base64_decode($text),
                MCRYPT_MODE_ECB,
                mcrypt_create_iv(mcrypt_get_iv_size(MCRYPT_RIJNDAEL_256, MCRYPT_MODE_ECB), MCRYPT_RAND)
            )
        );
    }

    public function formataDataBr($data) {
        return implode(preg_match("~\/~", $data) == 0 ? "/" : "-", array_reverse(explode(preg_match("~\/~", $data) == 0 ? "-" : "/", $data)));
    }

    public function formataDinheiro($num) {
        return 'R$' . number_format($num, 2, ',', '.');
    }
    
    public function formataPorcentagem($num) {
        return  $num . '%';
    }

    public function newCode($counterName, $size) {
        $nrseq = null;
        $connection = $this->entityManager->getConnection();
        $storedProcedure = new StoredProcedure($connection, 'NOVO_CODIGO');
        $storedProcedure->addParam(new Param('P_CONTADOR', Param::PARAM_INPUT, $counterName, Param::PARAM_TYPE_STR, strlen($counterName)));
        $storedProcedure->addParam(new Param('P_SEQUENCIAL', Param::PARAM_OUTPUT, $nrseq, Param::PARAM_TYPE_INT, 20));
        $result = $storedProcedure->execute();
        return substr($result['P_SEQUENCIAL'], strlen($result['P_SEQUENCIAL'])-$size);
    }
    
    public function procedAtualizaEstoque($CDFILIAL, $NRLANCESTQ) {
        $connection = $this->entityManager->getConnection();
        $storedProcedure = new StoredProcedure($connection, 'ATUALIZA_ESTOQUE_MANAGE');

        $dataAtual = date('d:m:Y');

        $storedProcedure->addParam($this->buildInputParam('P_CDFILILANC', $CDFILIAL));
        $storedProcedure->addParam($this->buildInputParam('V_DTINICIO', $dataAtual));
        $storedProcedure->addParam($this->buildInputParam('V_DTFIM', $dataAtual));
        $storedProcedure->addParam($this->buildInputParam('P_CDFILIAL', $CDFILIAL));
        $storedProcedure->addParam($this->buildInputParam('P_NRLANCESTQ', $NRLANCESTQ));
        
        $result = $storedProcedure->execute();
        return $result;
    }

    public function procedInsereItensDeComposicao($CDFILIAL, $NRLANCESTQ, $CDPRODUTO, $QTLANCTOEST, 
                                                $VRUNILANCTO, $TIPOMOVI, $CDALMOXARIFE, $CDLOCALESTOQ) {
        
        $connection = $this->entityManager->getConnection();
        $storedProcedure = new StoredProcedure($connection, 'INSERE_SUBPRODMAN');

        $dataAtual = date('d:m:Y');

        $storedProcedure->addParam($this->buildInputParam('P_CDFILIAL', $CDFILIAL));
        $storedProcedure->addParam($this->buildInputParam('P_LANCESTQ', $NRLANCESTQ));
        $storedProcedure->addParam($this->buildInputParam('P_TIPOOPER', '10'));
        $storedProcedure->addParam($this->buildInputParam('P_CDPRODUTO', $CDPRODUTO));
        $storedProcedure->addParam($this->buildInputParam('P_QTITEMPROD', $QTLANCTOEST));
        $storedProcedure->addParam($this->buildInputParam('P_VRTOTAITEM', $VRUNILANCTO * $QTLANCTOEST));
        $storedProcedure->addParam($this->buildInputParam('P_VRDESCITEM', '0'));
        $storedProcedure->addParam($this->buildInputParam('P_VRBRUTITEM', $VRUNILANCTO));
        $storedProcedure->addParam($this->buildInputParam('P_VRCONTITEM', '0'));
        $storedProcedure->addParam($this->buildInputParam('P_VRTOTALANCTO', $VRUNILANCTO * $QTLANCTOEST));
        $storedProcedure->addParam($this->buildInputParam('P_QTDELANCTO', $QTLANCTOEST));
        $storedProcedure->addParam($this->buildInputParam('P_TIPOMOVI', $TIPOMOVI));
        $storedProcedure->addParam($this->buildInputParam('P_INIVALID', $dataAtual));
        $storedProcedure->addParam($this->buildInputParam('P_CDPRODESTO', $CDPRODUTO));
        $storedProcedure->addParam($this->buildInputParam('P_NRPEDIDO', null));
        $storedProcedure->addParam($this->buildInputParam('P_NRITEMPED', null));
        $storedProcedure->addParam($this->buildInputParam('P_IDTPCOMPPROD', 'F'));
        $storedProcedure->addParam($this->buildInputParam('P_CDALMOXARIFE', $CDALMOXARIFE));
        $storedProcedure->addParam($this->buildInputParam('P_CDLOCALESTOQ', $CDLOCALESTOQ));
        
        $result = $storedProcedure->execute();
        return $result;
    }

    private function buildInputParam ($paramName, $paramValue) {
        return new Param($paramName, Param::PARAM_INPUT, $paramValue, Param::PARAM_TYPE_STR, strlen($paramValue));
    }

    public function formataHora4Digitos($horario) {
        $hora = substr($horario, 0,2);
        $minutos = substr($horario, 2,5);

        return $hora . ':' . $minutos;
    }

    public function formataHora4DigitosReverso($horario) {
        $hora = substr($horario, 0,2);
        $minutos = substr($horario, 3,5);

        return $hora . $minutos;
    }

    public static function nextNDay($dias) {
        return date("d-m-Y",mktime(0, 0, 0, date("m"), date("d")+$dias, date("y")));
    }

    public function getParams ($params) {
    	
        $treatedParams = array();
        foreach ($params as $param) {
            
             if (!empty($param['value'])) {
	            if ($param['value'] == '=') {
	                $param['value'] = '';
	            }
            }
            $treatedParams[$param['columnName']] = $param['value'];
        }

        return $treatedParams;
    }

    public function formatDate($date) {
        //Formata o dia e o mes com dois algarismos e retorna a data no formato brasileiro
        $date = explode("/",$date);        
        return str_pad($date[0],2,"0",STR_PAD_LEFT) .'/'.str_pad($date[1],2,"0",STR_PAD_LEFT).'/'.$date[2];
    }

    public function directoryCheck($directory) {
        
        if ( !is_dir($directory) && ! @mkdir($directory, 0777, true)) {
            throw new \InvalidArgumentException(sprintf(
                'O diretório "%s" não pode ser criado.',
                $directory
            ));
        }

        if ( ! is_writable($directory)) {
            throw new \InvalidArgumentException(sprintf(
                'O diretório "%s" não pode tem permissão de escrita.',
                $directory
            ));
        }
    }

    // Use it to get Zeedhi's Request's objects as array objects
    
    // DataSet  -> array of Zeedhi Rows
    // Row      -> Row  
    // Filter   -> associative array like an Zeedhi Row 
    
    public function getParamsAsArray($obj) {
        if($obj instanceof Request\DataSet) {
            // returns An Array
            return $obj->getDataSet()->getRows();
        } elseif ($obj instanceof Request\Row) {
            return $obj->getRow();
        } elseif ($obj instanceof Request\Filter) {
            $arr = array();
            foreach ($obj->getFilterCriteria()->getConditions() as $condition) {
                $arr[ $condition['columnName'] ] = $condition['value'];  
            }
            return $arr;
        }
    }
    
    public function fixFilterTCD ($conditions, $conditionName) {
        if(array_key_exists( $conditionName, $conditions) ) {
            $conditions['T'.$conditionName] = 'F';
        } else {
            $conditions['T'.$conditionName] = 'T';
            $conditions[$conditionName] = null;
        }
    }
    
    
    // Testing
    public function getZeedhiRowAsArray(Request\DataSet $obj) {
        $arr = [];
        foreach ($obj as $key => $value) {
            $arr[$key] = $value;
        }
        return $arr;
        
    }
    
    // Converte TColor para Hexadecimal
    public static function convertTColortoHexa($TColor) {
        $stringColor = '';
        $r = $TColor & 0xFF;
        $g = ($TColor >> 8) & 0xFF;
        $b = ($TColor >> 16) & 0xFF;
        $Hexa = ((1 << 24) + ($r << 16) + ($g << 8) + $b);
        $stringHexa = dechex($Hexa);
        $stringColor = (string)$stringHexa;
        $color = '#'.(substr($stringColor, 1));
        }

    // Converte rgb para Tcolor
    public static function RGBToTColor($r, $g, $b) {
        $Tcolor = $r | ($g << 8) | ($b << 16);
        return $Tcolor;
    }
    
    // HEXDEC converte de hexa para decimal :)
    public static function hexToRGB($hex) {
       $hex = str_replace("#", "", $hex);
    
       if(strlen($hex) == 3) {
          $r = hexdec(substr($hex,0,1).substr($hex,0,1));
          $g = hexdec(substr($hex,1,1).substr($hex,1,1));
          $b = hexdec(substr($hex,2,1).substr($hex,2,1));
       } else {
          $r = hexdec(substr($hex,0,2));
          $g = hexdec(substr($hex,2,2));
          $b = hexdec(substr($hex,4,2));
       }
       $RGB = array($r, $g, $b);
       return $RGB; // returns an array with the rgb values
    }
    
    // formata data para um valor inteiro
    public static function formatDateToInteger($data) {
        return intVal(implode('', array_reverse(explode('/', $data)))); 
    }
    
    public static function Ajusta($stString, $inTamStr) {
        $result = $stString;
       if(strlen($stString) != $inTamStr) {
    		if(!empty($stString)) {
    		  for($i = 0;$i < strlen($stString);$i++) {
    			  if (!(is_numeric($stString[$i])) && ($stString[$i] != '.')) {break;}
    			}
    		   $result = $stString;
    		  for($inInString = 0; $inInString < ($inTamStr - strlen($stString)); $inInString++) {
    		  	$result = "0".$result;
    		  }
    		}
    	}
    	return $result;
    }
    
    public static function Pot($Numero, $Expoente) {
       return exp($Expoente * log($Numero));
    }
    
    // arredondamento no padrão da bematech
    public static function RoundDec($valor, $decimais) {
    	$ParteInt = intval($valor);
    	if($decimais > 9)
    		$decimais = 9;
    	$P      = round(self::Pot(10, $decimais));      //    100000
    	$Aux 	 = round(abs($valor-floor($valor)) * $P);   //    36549
    	if($Aux == $P) {      
    		if($ParteInt < 0) {
    			$ParteInt--;
    		}
    		else{
    			$ParteInt++;
    		}
    		$Aux = 0;
    	}
    	$decimalSeparator = self::Ajusta($Aux, $decimais) == null  ? '' : '.'; 
    	if (($valor < 0) && ($ParteInt == 0)) {
    		return '-'.$ParteInt.$decimalSeparator.(self::Ajusta($Aux, $decimais)); 
    	} else{
    		return $ParteInt.$decimalSeparator.(self::Ajusta($Aux, $decimais));      //    58, 36549
    	} 
    }
    
    public function buildParams ($key, $conditions) {
        if(!array_key_exists($key, $conditions) || !count($conditions[$key])) {
            $conditions[$key] = '';
            $conditions['T'.$key] = 'T';
        } else {
            $conditions[$key] = implode('_', $conditions[$key]);
            $conditions['T'.$key] = '';
        }
        return $conditions;
    }
    
    public function buildParamsNotArray ($key, $conditions) {
        if(!array_key_exists($key, $conditions)) {
            $conditions[$key] = '';
            $conditions['T'.$key] = 'T';
        } else {
            $conditions['T'.$key] = '';
        }
        return $conditions;
    }
    
    public function setImageForSave (Row $row, $columnName) {
        if(empty($row[$columnName])) {
            $row->set($columnName, NULL);
        } else {
            if (is_string($row[$columnName])) {
                $row->set($columnName, str_replace(self::ABSOLUTE_PATH, "", $row[$columnName]));
            } else {
                if(count($row[$columnName]) !== 1) {
                    throw new \Exception('Só é possível salvar apenas uma imagem por vez.');
                } else {
                    $row->set($columnName, $row[$columnName][0]['path']);
                }
            }
        }
    }
    
    // Returns the names of the conditions that use the operator LIKE
    public function getLikeConditions($request) {
        $conditions = $request->getFilterCriteria()->getConditions();
        $result = array();
        foreach ($conditions as $condition) {
            if($condition['operator'] === 'LIKE_ALL' || $condition['operator'] === 'LIKE_I' || $condition['operator'] === 'MAPPED_LIKE_ALL' )
                array_push($result, $condition);
        }
        return $result;
    }
    
    // Create a new Zeedhi's FilterCriteria's Request object with the conditions array.
    // Obs: all conditions will have the operator equal ('=') in it.
    public function createNewZeedhiFilterRequestObject($request, $dataSourceName, $conditions) {
        foreach($conditions as $key => $value) {
            $dataSourceFilter[] = $condition = array(
                "columnName" => $key,
                "operator"=> "=",
                "value"=> $value
            );
        }
        
        return new DTO\Request\Filter(
            new FilterCriteria(
                $dataSourceName,
                $dataSourceFilter,
                $request->getFilterCriteria()->getPage(),
                $request->getFilterCriteria()->getPageSize()
            ),
            $request->getMethod(),
            $request->getRoutePath(),
            $request->getUserId()
        );
    }
    
    public function validarCPF($cpf = null) {

    	// Verifica se um número foi informado
    	if(empty($cpf)) {
    		return false;
    	}
    
    	// Elimina possivel mascara
    	$cpf = preg_replace("/[^0-9]/", "", $cpf);
    	$cpf = str_pad($cpf, 11, '0', STR_PAD_LEFT);
    	
    	// Verifica se o numero de digitos informados é igual a 11 
    	if (strlen($cpf) != 11) {
    		return false;
    	}
    	// Verifica se nenhuma das sequências invalidas abaixo 
    	// foi digitada. Caso afirmativo, retorna falso
    	else if ($cpf == '00000000000' || 
    		$cpf == '11111111111' || 
    		$cpf == '22222222222' || 
    		$cpf == '33333333333' || 
    		$cpf == '44444444444' || 
    		$cpf == '55555555555' || 
    		$cpf == '66666666666' || 
    		$cpf == '77777777777' || 
    		$cpf == '88888888888' || 
    		$cpf == '99999999999') {
    		return false;
    	 // Calcula os digitos verificadores para verificar se o
    	 // CPF é válido
    	 } else {   
    		
    		for ($t = 9; $t < 11; $t++) {
    			
    			for ($d = 0, $c = 0; $c < $t; $c++) {
    				$d += $cpf{$c} * (($t + 1) - $c);
    			}
    			$d = ((10 * $d) % 11) % 10;
    			if ($cpf{$c} != $d) {
    				return false;
    			}
    		}
    
    		return true;
    	}
    }
    
    public function validarCNPJ($cnpj) {
    	$cnpj = preg_replace('/[^0-9]/', '', (string) $cnpj);
    	// Valida tamanho
    	if (strlen($cnpj) != 14)
    		return false;
    	// Valida primeiro dígito verificador
    	for ($i = 0, $j = 5, $soma = 0; $i < 12; $i++)
    	{
    		$soma += $cnpj{$i} * $j;
    		$j = ($j == 2) ? 9 : $j - 1;
    	}
    	$resto = $soma % 11;
    	if ($cnpj{12} != ($resto < 2 ? 0 : 11 - $resto))
    		return false;
    	// Valida segundo dígito verificador
    	for ($i = 0, $j = 6, $soma = 0; $i < 13; $i++)
    	{
    		$soma += $cnpj{$i} * $j;
    		$j = ($j == 2) ? 9 : $j - 1;
    	}
    	$resto = $soma % 11;
    	return $cnpj{13} == ($resto < 2 ? 0 : 11 - $resto);
    }
    
}